from pathlib import Path


class Constants:
    coding = 'utf-8'

    bt7086_url = 'http://bt7086.org/pw'

    user_agent = 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) ' \
                 'AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.97 Mobile Safari/537.36'

    @staticmethod
    def project_root_path():
        return str(Path(__file__).parent.parent)



import subprocess
import logging
import re
import sys
from config import YAMLConfig, Constants
import aria2p

logger = logging.getLogger(__name__)


class Aria2:
    _instance = None

    def __init__(self):
        if Aria2._instance is not None:
            raise Exception('Aria2 Singleton creation failed, there has a instance already.')
        else:
            self._start_aria2_cmd = None
            self._aria2p = None
            Aria2._instance = self

    @staticmethod
    def get_instance():
        if Aria2._instance is None:
            Aria2()
        return Aria2._instance

    @staticmethod
    def is_installed():
        output = subprocess.check_output('aria2c -v', shell=True).decode(Constants.coding)
        version = re.search(r'(?<=aria2 version )\d\.\d{1,2}\.\d', output)
        if version is None:
            return False
        else:
            logger.debug('Aria2 is installed.')
            return True

    @property
    def start_aria2_cmd(self):
        return self._start_aria2_cmd

    @start_aria2_cmd.setter
    def start_aria2_cmd(self, cmd):
        self._start_aria2_cmd = cmd

    @property
    def pid(self):
        cmd = 'ps -ax | grep aria2c'
        out = subprocess.check_output(cmd, shell=True).decode(Constants.coding)
        processes = out.split('\n')
        # logger.debug('processes: %s' % processes)
        for p in processes:
            if self.start_aria2_cmd is not None and p.find(self.start_aria2_cmd) != -1:
                pid = re.sub(r'(?<=\d{3}) .* %s' % self.start_aria2_cmd, 'AAAAAA', p)
                pid = re.search(r'\d+(?=AAAAAA)', pid)
                if pid is not None:
                    pid = pid.group(0)
                    logging.debug('PID of Aria2: %s' % pid)
                    return pid
        return False

    def start(self):
        # check the status of aria2c installation
        if Aria2.is_installed() is False:
            logger.error('Aria2 is not installed.')
            sys.exit(0)
        # not start repeatedly if aria2c has already started
        if self.pid:
            logger.debug('aria2c is running and it will return aria2p directly')
            return self.aria2p
        # start
        config = YAMLConfig.get()
        cmd = 'aria2c --conf-path=%s -D' % config['aria2']['config']
        logger.debug('command of starting aria2: %s' % cmd)
        logger.info('Starting Aria2...')
        out = subprocess.check_output(cmd, shell=True).decode(Constants.coding)
        logger.debug('outputs of starting aria2: %s' % out)
        if out != '':
            logger.error('aria2 start failed: %s' % out)
            raise Exception('aria2 start failed: %s' % out)
        else:
            self.start_aria2_cmd = cmd
            if not self.pid:
                raise Exception('Aria2c has gone away')
        return self.aria2p

    def shutdown(self):
        pid = self.pid
        if pid:
            logger.info('Shutting down Aria2c(PID: %s)...' % pid)
            self._aria2p = None
            subprocess.call('kill %s' % pid, shell=True)

    @property
    def aria2p(self):
        if self._aria2p is None:
            if not self.pid:
                logger.error('Aria2p creation failed: Aria2c is not running.')
                return
            aria2_client = aria2p.Client(host='http://localhost', port=6800)
            self._aria2p = aria2p.API(aria2_client)
        return self._aria2p



import m3u8
import re
import logging
import time

logger = logging.getLogger(__name__)


class Parser(object):

    @staticmethod
    def m3u8_url_to_download_link_array(url):
        logger.info('\'%s\' is loading...' % url)
        start_time = time.time()
        m3u8_obj = m3u8.load(url)
        duration = (time.time() - start_time) * 1000
        logger.info('\'%s\' is loaded, the process costs %.2f ms' % (url, duration))
        segments = []
        for s in m3u8_obj.segments:
            segments.append(s.uri)
        logger.debug('\'%s\' is converted to %s' % (url, segments))
        if len(segments) == 0:
            logger.error('\'%s\' can\'t would be skipped because it couldn\'t be loaded by m3u8 module.' % url)
            return None
        return segments

    @staticmethod
    def check_m3u8s(m3u8s):
        for i in m3u8s:
            if 'name' not in i:
                raise KeyError('Name for every item of m3u8_list is required.')
            if 'url' not in i:
                raise KeyError('Url for every item of m3u8_list is required.')
            if re.search(r'^https?://.*\.m3u8$', i['url']) is None:
                raise ValueError('Url format is invalid, url: %s' % i['url'])
        logger.debug('Format of m3u8s is acceptable.')
        return True

    @staticmethod
    def parse(m3u8s):
        Parser.check_m3u8s(m3u8s)
        download_list = []
        for m in m3u8s:
            name = m['name']
            url = m['url']
            segments = Parser.m3u8_url_to_download_link_array(url)
            if segments is not None:
                download_list.append({
                    'name': name,
                    'segments': segments
                })
        logger.debug('parsing results of m3u8 list: %s' % download_list)
        return download_list



import click
import logging
from m3u8_dl.downloader import Downloader
from collector import Collector
from version import __version__ as ver
import logging

logger = logging.getLogger(__name__)

banner = """
       ____                           
      / __ \ ___   ____ _ _____ __  __
     / /_/ // _ \ / __ `// ___// / / /
    / _, _//  __// /_/ /(__  )/ /_/ / 
   /_/ |_| \___/ \__,_//____/ \__, /  
                             /____/              
   :: Running Reasy v%s ::
    """ % ver


def echo_banner():
    click.echo(banner)


@click.group()
@click.version_option(None,
                      '-v', '--version',
                      message='reasy version %s' % ver)
def cli():
    pass


def verbose_callback(ctx, param, value):
    if value:
        click.echo('Enables verbose.')
        logging.basicConfig(
            format='%(asctime)s [%(levelname)s] %(message)s (%(filename)s:%(lineno)s)',
            datefmt='%Y-%m-%d %H:%M:%S',
            level=logging.DEBUG,
        )
    else:
        logging.basicConfig(
            format='%(asctime)s [%(levelname)s] %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S',
            level=logging.INFO,
        )


@cli.command()
@click.option('--no-session',
              is_flag=True,
              help='Disable session, which may cause repeated download.')
@click.option('--max-dl-num', '-m',
              default=30,
              type=int,
              help='Maximum download number per performing, default: 30.')
@click.option('--verbose',
              is_flag=True,
              expose_value=False,
              is_eager=True,
              callback=verbose_callback,
              help='Enables verbose.')
def run(no_session, max_dl_num):
    echo_banner()
    session = not no_session
    # if no_session:
    #     click.echo('Session has been disabled.')
    dl = Downloader(session=session)
    m3u8s = Collector.get_m3u8s(session, max_dl_num)
    dl.download(m3u8s)



import logging
from config import YAMLConfig, Constants
import os
import base64

logger = logging.getLogger(__name__)


class Session:

    @staticmethod
    def read():
        if not YAMLConfig.session_enabled():
            logger.debug('session is disabled.')
            return None
        if not os.path.isfile(YAMLConfig.session_path()):
            logger.error('session file not exists, please create it before using.')
            raise FileNotFoundError('session file not exists.')
        with open(YAMLConfig.session_path(), 'rb') as f:
            encoded_bytes = f.read()
            content = base64.b64decode(encoded_bytes).decode(Constants.coding)
            logger.debug('content of session: %s' % content)
            return content

    @staticmethod
    def save(name):
        if not YAMLConfig.session_enabled():
            logger.debug('session is disabled.')
            return None
        if not os.path.isfile(YAMLConfig.session_path()):
            logger.error('session file not exists, please create it before using.')
            raise FileNotFoundError('session file not exists.')
        with open(YAMLConfig.session_path(), 'wb') as f:
            encoded_bytes = name.encode(Constants.coding)
            f.write(base64.b64encode(encoded_bytes))
            return True

    @staticmethod
    def clear():
        Session.save('')



import yaml
import logging
import sys
import os

logger = logging.getLogger(__name__)


class YAMLConfig:
    _config = None

    @staticmethod
    def get():
        if YAMLConfig._config is None:
            try:
                with open(os.getenv('REASY_CONFIG', '/.reasy/config.yaml'), 'r') as file:
                    YAMLConfig._config = yaml.safe_load(file)
                    logger.debug('config.yaml: %s' % YAMLConfig._config)
            except FileNotFoundError:
                logging.error('Config is not existed.')
                sys.exit(0)
        return YAMLConfig._config

    @staticmethod
    def session_enabled():
        return YAMLConfig.get()['session']['enabled']

    @staticmethod
    def session_path():
        return YAMLConfig.get()['session']['path']
